//--------------------------------------------------------------------------------------
// File: main.fx
//
// The effect file for the main sample.  
// 
// Copyright (c) Takashi Imagire. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float	 g_fTime;					// App's time in seconds
float	 g_fRadius = 5.0;			// 
float4x4 g_mWorld;					// World matrix for object
float4x4 g_mWorldViewProjection;	// World * View * Projection matrix
float4x4 g_mShadowViewProjection;
float4x4 g_mShadowMatrix;
float4   g_Offset[12];
float4   g_vLightPos;

float INV_TEX_WIDTH;
float INV_TEX_HEIGHT;
float4x4 g_mShadowViewProjection0;
float4x4 g_mShadowViewProjection1;
float4x4 g_mShadowMatrix0;
float4x4 g_mShadowMatrix1;

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
texture Texture;
sampler Sampler = 
sampler_state
{
    Texture = <Texture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture ShadowTex;
sampler ShadowSampler = 
sampler_state
{
    Texture = <ShadowTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture ShadowTex0;
sampler ShadowSampler0 = 
sampler_state
{
    Texture = <ShadowTex0>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture ShadowTex1;
sampler ShadowSampler1 = 
sampler_state
{
    Texture = <ShadowTex1>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture PointTex;
sampler PointSampler = 
sampler_state
{
    Texture = <PointTex>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = POINT;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture PositionTex0;
texture PositionTex1;
texture PositionTex2;
texture PositionTex3;
texture PositionTex4;

sampler PositionSampler0 = sampler_state {Texture = <PositionTex0>; MinFilter = LINEAR; MagFilter = LINEAR; MipFilter = NONE; AddressU = Clamp; AddressV = Clamp;};
sampler PositionSampler1 = sampler_state {Texture = <PositionTex1>; MinFilter = LINEAR; MagFilter = LINEAR; MipFilter = NONE; AddressU = Clamp; AddressV = Clamp;};
sampler PositionSampler2 = sampler_state {Texture = <PositionTex2>; MinFilter = LINEAR; MagFilter = LINEAR; MipFilter = NONE; AddressU = Clamp; AddressV = Clamp;};
sampler PositionSampler3 = sampler_state {Texture = <PositionTex3>; MinFilter = LINEAR; MagFilter = LINEAR; MipFilter = NONE; AddressU = Clamp; AddressV = Clamp;};
sampler PositionSampler4 = sampler_state {Texture = <PositionTex4>; MinFilter = LINEAR; MagFilter = LINEAR; MipFilter = NONE; AddressU = Clamp; AddressV = Clamp;};

//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position  : POSITION;
    float4 Pos       : TEXCOORD0;
};



//-----------------------------------------------------------------------------
// Name: CreateShadowMapVS
// Type: Vertex shader                                      
// Desc: standard transform and lighting.
//-----------------------------------------------------------------------------
VS_OUTPUT CreateShadowMapVS( float4 vPos : POSITION, 
                    float3 vNormal : NORMAL,
                    float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
    float4 pos;
  
    // tranform vertex position into screen space
    pos = mul(vPos, g_mWorldViewProjection );
    Output.Position = pos;
    
    // ˉeԂ̍WleNX`Wɂēn
    Output.Pos      = pos;
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: SimplePS
// Type: Pixel shader
// Desc: This shader simply outputs the pixel's color 
//-----------------------------------------------------------------------------
float4 CreateShadowMapPS( VS_OUTPUT In ) : COLOR
{ 
    return In.Pos.z / In.Pos.w;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique CreateShadowMap
{
    pass P0
    {
        VertexShader = compile vs_1_1 CreateShadowMapVS();
        PixelShader  = compile ps_2_0 CreateShadowMapPS();
    }
}



//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct POSITION_FRAGMENT
{
    float4 Position   : POSITION;
    float4 TextureUV  : TEXCOORD0;
    float4 Pos0       : TEXCOORD1;
    float4 Pos1       : TEXCOORD2;
    float4 ShadowMap0 : TEXCOORD3;
    float4 ShadowMap1 : TEXCOORD4;
};


//-----------------------------------------------------------------------------
// Name: PositionMapVS
// Type: Vertex shader                                      
// Desc: standard transform and lighting.
//-----------------------------------------------------------------------------
POSITION_FRAGMENT PositionMapVS( float4 vPos : POSITION, 
                    float3 vNormal : NORMAL,
                    float2 vTexCoord0 : TEXCOORD0 )
{
    POSITION_FRAGMENT Output;
    float4 pos;
  
    // tranform vertex position into screen space
    Output.Position  = mul(vPos, g_mWorldViewProjection );
    
    // zJƂ݂ȂƂ̎ˉeԂւ̃}bsO
    Output.Pos0       = mul(vPos, g_mShadowViewProjection0 );
    Output.Pos1       = mul(vPos, g_mShadowViewProjection1 );
    
    // ɃoCAXςāAeNX`̍WnɑΉ
    Output.ShadowMap0 = mul(vPos, g_mShadowMatrix0 );
    Output.ShadowMap1 = mul(vPos, g_mShadowMatrix1 );

    // eNX`WPɃRs[
    Output.TextureUV.xy = vTexCoord0;
    Output.TextureUV.z = 0;
    
    // [JWnł̌vZ
    pos = g_vLightPos - vPos;
    Output.TextureUV.w = max( dot(normalize(pos).xyz, vNormal), 0);

    return Output;    
}

//-----------------------------------------------------------------------------
// Name: PositionMapPS
// Type: Pixel shader
// Desc: 
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 DarkOrSine : COLOR0;
    float4 Decale     : COLOR1;
};

PS_OUTPUT PositionMapPS( POSITION_FRAGMENT In )
{
	PS_OUTPUT Output;
	
	float shadow_map0 = tex2Dproj( ShadowSampler0, In.ShadowMap0 ).x;
	float shadow_map1 = tex2Dproj( ShadowSampler1, In.ShadowMap1 ).x;
	
	float depth0 = In.Pos0.z / In.Pos0.w - 0.003;
	float depth1 = In.Pos1.z / In.Pos1.w - 0.003;
	
	float shade0 = (( shadow_map0 < depth0 ) ? 0 : 1);
	float shade1 = (( shadow_map1 < depth1 ) ? 0 : 1);
	
	Output.DarkOrSine.rgb = (1.0-shade0) * (1.0-shade1); // SɈÂ
	Output.DarkOrSine.a   = shade0 * shade1;             // Sɖ邢

	// eNX`ƊgUo͂
	Output.Decale.xyz = tex2D( Sampler, In.TextureUV ).xyz;
	Output.Decale.w   = In.TextureUV.w;
	
	return Output;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique CreatePositionMap
{
    pass P0
    {
        VertexShader = compile vs_1_1 PositionMapVS();
        PixelShader  = compile ps_2_0 PositionMapPS();
    }
}




//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct FRAGMENT_SMALLBUFFER
{
    float4 Position  : POSITION;
    float2 Tex0      : TEXCOORD0;
    float2 Tex1      : TEXCOORD1;
    float2 Tex2      : TEXCOORD2;
    float2 Tex3      : TEXCOORD3;
};


//-----------------------------------------------------------------------------
// Name: CreateSmallBufferVS
// Type: Vertex shader                                      
// Desc: 
//-----------------------------------------------------------------------------
FRAGMENT_SMALLBUFFER CreateSmallBufferVS( float4 vPos : POSITION, 
                    float2 vTexCoord0 : TEXCOORD0 )
{
    FRAGMENT_SMALLBUFFER Output;
  
    // eNX`Ŵ܂܃Rs[
    Output.Position  = vPos;
    
    // eNX`WPɃRs[
    Output.Tex0.xy = vTexCoord0 + float2( 1.0 * INV_TEX_WIDTH, 1.0 * INV_TEX_HEIGHT );
    Output.Tex1.xy = vTexCoord0 + float2( 3.0 * INV_TEX_WIDTH, 1.0 * INV_TEX_HEIGHT );
    Output.Tex2.xy = vTexCoord0 + float2( 1.0 * INV_TEX_WIDTH, 3.0 * INV_TEX_HEIGHT );
    Output.Tex3.xy = vTexCoord0 + float2( 3.0 * INV_TEX_WIDTH, 3.0 * INV_TEX_HEIGHT );
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: CreateSmallBufferPS
// Type: Pixel shader
// Desc: 
//-----------------------------------------------------------------------------
float4 CreateSmallBufferPS( FRAGMENT_SMALLBUFFER In ) : COLOR
{
	float4 c0 = tex2D( Sampler, In.Tex0 );
	float4 c1 = tex2D( Sampler, In.Tex1 );
	float4 c2 = tex2D( Sampler, In.Tex2 );
	float4 c3 = tex2D( Sampler, In.Tex3 );
	
    return 0.25 * (c0 + c1 + c2 + c3);
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique CreateSmallBuffer
{
    pass P0
    {
        VertexShader = compile vs_1_1 CreateSmallBufferVS();
        PixelShader  = compile ps_2_0 CreateSmallBufferPS();
    }
}




//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct FRAGMENT_COMPUTE
{
    float4 Position  : POSITION;
    float2 Tex0      : TEXCOORD0;
    float2 Tex1      : TEXCOORD1;
    float2 Tex2      : TEXCOORD2;
    float2 Tex3      : TEXCOORD3;
    float2 Tex4      : TEXCOORD4;
};


//-----------------------------------------------------------------------------
// Name: ComputeShadowVS
// Type: Vertex shader                                      
// Desc: 
//-----------------------------------------------------------------------------
FRAGMENT_COMPUTE ComputeShadowVS( float4 vPos : POSITION, 
                    float2 vTexCoord0 : TEXCOORD0 )
{
    FRAGMENT_COMPUTE Output;
  
    // tranform vertex position into screen space
    Output.Position  = vPos;
    
    // eNX`WPɃRs[
    Output.Tex0.xy = vTexCoord0;
    Output.Tex1.xy = vTexCoord0;
    Output.Tex2.xy = vTexCoord0;
    Output.Tex3.xy = vTexCoord0;
    Output.Tex4.xy = vTexCoord0;
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: ComputeShadowPS
// Type: Pixel shader
// Desc: 
//-----------------------------------------------------------------------------
float4 ComputeShadowPS( FRAGMENT_COMPUTE In ) : COLOR
{
	float4 c0 = tex2D( PositionSampler0, In.Tex0 );
	float4 c1 = tex2D( PositionSampler1, In.Tex1 );
	float4 c2 = tex2D( PositionSampler2, In.Tex2 );
	float4 c3 = tex2D( PositionSampler3, In.Tex3 );
	float4 c4 = tex2D( PositionSampler4, In.Tex4 );
	
	const float ratio = 0.5;
	float b = c0.r
			 + (ratio) * c1.r
			 + (ratio*ratio) * c2.r
			 + (ratio*ratio*ratio) * c3.r
			 + (ratio*ratio*ratio*ratio) * c4.r;
			 
	float w = c0.a
			 + (ratio) * c1.a
			 + (ratio*ratio) * c2.a
			 + (ratio*ratio*ratio) * c3.a
			 + (ratio*ratio*ratio*ratio) * c4.a;

	float nb = saturate((1-w)*b);
	float nw = saturate((1-b)*w);
	
    return nw / (nb + nw);
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique ComputeShadow
{
    pass P0
    {
        VertexShader = compile vs_1_1 ComputeShadowVS();
        PixelShader  = compile ps_2_0 ComputeShadowPS();
    }
}




//-----------------------------------------------------------------------------
// Name: FinalPS
// Type: Pixel shader
// Desc: 
//-----------------------------------------------------------------------------
float4 FinalPS( float2 vTexCoord0 : TEXCOORD0 ) : COLOR
{
	const float ambient = 0.3;
	
	float4 color = tex2D(       Sampler, vTexCoord0 );
	float  shade = tex2D( ShadowSampler, vTexCoord0 ).x;

	shade = (1.0-ambient) * shade * color.w + ambient;
	
    return shade * color;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique Final
{
    pass P0
    {
        PixelShader  = compile ps_2_0 FinalPS();
    }
}






